from contextlib import contextmanager
from typing import Optional, List

from .access_rules import AccessRuleImpl
from .chart import ChartImpl, Chart
from .chat import ChatImpl, Chat
from .history import HistoryImpl, History
from .query import QueryImpl, Query
from .database import DatabaseImpl, Database
from .semantic_context import SemanticContextImpl, SemanticContext
from .user import UserImpl
from .user.user_static import User
from .waii_http_client import WaiiHttpClient
import importlib.metadata
from .my_pydantic import BaseModel

GET_MODELS_ENDPOINT = "get-models"


class GetModelsRequest(BaseModel):
    pass


class ModelType(BaseModel):
    name: str
    description: Optional[str]
    vendor: Optional[str]


class GetModelsResponse(BaseModel):
    models: Optional[List[ModelType]]


class Waii:

    def __init__(self, initialize_legacy_fields: bool = False):
        self.history = None
        self.query = None
        self.database = None
        self.semantic_context = None
        self.chat = None
        self.chart = None
        self.user = None
        self.initialize_legacy_fields = initialize_legacy_fields
        self.http_client = None

    def initialize(self, url: str = "https://tweakit.waii.ai/api/", api_key: str = ""):
        http_client = WaiiHttpClient(url, api_key)
        self.http_client = http_client
        self.history = HistoryImpl(http_client)
        self.query = QueryImpl(http_client)
        self.database = DatabaseImpl(http_client)
        self.semantic_context = SemanticContextImpl(http_client)
        self.chat = ChatImpl(http_client)
        self.chart = ChartImpl(http_client)
        self.user = UserImpl(http_client)
        self.access_rule = AccessRuleImpl(http_client)

        if self.initialize_legacy_fields:
            self.History = self.history
            self.Query = self.query
            self.Database = self.database
            self.SemanticContext = self.semantic_context
            self.Chart = self.chart
            self.Chat = self.chat
            self.User = self.user
            Query.http_client = http_client
            History.http_client = http_client
            Database.http_client = http_client
            SemanticContext.http_client = http_client
            User.http_client = http_client
            Chat.http_client = http_client
            Chart.http_client = http_client


        conns = self.database.get_connections().connectors
        if len(conns) > 0:
            self.database.activate_connection(conns[0].key)

    @staticmethod
    def version():
        return importlib.metadata.version('waii-sdk-py')

    def get_models(self, params: GetModelsRequest = GetModelsRequest()) -> GetModelsResponse:
        return self.http_client.common_fetch(
            GET_MODELS_ENDPOINT, params.__dict__, GetModelsResponse
        )

    @contextmanager
    def impersonate_user(self, user_id: str):
        try:
            self.http_client.set_impersonate_user_id(user_id)
            yield
        finally:
            self.clear_impersonation()

    def set_impersonate_user(self, user_id: str):
        self.http_client.set_impersonate_user_id(user_id)

    def clear_impersonation(self):
        self.http_client.set_impersonate_user_id('')

WAII = Waii(True)



import unittest

from tests.common_test_utils import load_db_conn1, load_db_conn2, connect_db, org_exists, tenant_exists, user_exists
from waii_sdk_py.query import QueryGenerationRequest
from waii_sdk_py.semantic_context import ModifySemanticContextRequest, SemanticStatement
from waii_sdk_py.user import Organization, CreateOrganizationRequest, Tenant, CreateTenantRequest, User, \
    CreateUserRequest, WaiiRoles, ListUsersRequest, DeleteUserRequest, DeleteTenantRequest
from waii_sdk_py.waii_sdk_py import Waii, WAII


class MultiTenantSemanticContextTest(unittest.TestCase):
    def setUp(self):
        self.movie_conn = load_db_conn1()
        self.album_conn = load_db_conn2()

        waii_client = Waii()
        waii_client.initialize(url="http://localhost:9859/api/")
        self.waii_client = waii_client
        self.waii_client.database.activate_connection(self.movie_conn.key)

        self.setup_users()

        # add the database connections to users
        for impersonate_user in ['multi-tenant-semantic-context-test-user1', 'multi-tenant-semantic-context-test-user2',
                                 '']:
            connect_db(self.movie_conn, impersonate_user=impersonate_user)
            connect_db(self.album_conn, impersonate_user=impersonate_user)

        # setup semantic context
        self.create_global_semantic_statements()

        # before we start, delete all the semantic rules
        self.delete_all_semantic_rules("multi-tenant-semantic-context-test-user1")
        self.delete_all_semantic_rules("multi-tenant-semantic-context-test-user2")

        self.create_semantic_rules_for_user1()
        self.create_semantic_rules_for_user2()

    def delete_all_semantic_rules(self, user_id):
        with self.waii_client.impersonate_user(user_id):
            response = self.waii_client.semantic_context.get_semantic_context().semantic_context
            for semantic in response:
                if semantic.user_id == user_id:
                    self.waii_client.semantic_context.modify_semantic_context(
                        ModifySemanticContextRequest(deleted=[semantic.id]))
            self.waii_client.clear_impersonation()

    def setup_users(self):
        client = self.waii_client

        # first list users under the new org
        response = client.user.list_users(ListUsersRequest(lookup_org_id="multi-tenant-semantic-context-test"))
        for user in response.users:
            client.user.delete_user(DeleteUserRequest(id=user.id))

        # delete the tenant
        response = client.user.list_tenants(ListUsersRequest(lookup_org_id="multi-tenant-semantic-context-test"))
        for tenant in response.tenants:
            client.user.delete_tenant(DeleteTenantRequest(id=tenant.id))

        # Create organization
        org_id = "multi-tenant-semantic-context-test"
        if not org_exists(client, org_id):
            org = Organization(id=org_id, name="Multi-tenant Semantic Context Test Org")
            client.user.create_org(CreateOrganizationRequest(organization=org))

        # Create tenant
        tenant_id = "test-tenant"
        if not tenant_exists(client, org_id, tenant_id):
            tenant = Tenant(id=tenant_id, name="Test Tenant", org_id=org_id)
            client.user.create_tenant(CreateTenantRequest(tenant=tenant))

        # Create users
        users = [
            {"id": "multi-tenant-semantic-context-test-user1", "name": "User 1"},
            {"id": "multi-tenant-semantic-context-test-user2", "name": "User 2"}
        ]

        for user_info in users:
            if not user_exists(client, org_id, user_info["id"]):
                # create user with WAII_USER role
                user = User(id=user_info["id"], name=user_info["name"], tenant_id=tenant_id, org_id=org_id,
                            roles=[WaiiRoles.WAII_USER])
                client.user.create_user(CreateUserRequest(user=user))
        print("Hierarchy creation completed.")

    def create_global_semantic_statements(self):
        request = ModifySemanticContextRequest(updated=[SemanticStatement(
            id="global-semantic-statement-1",
            statement="All the returned query must project column 'WAII_TEST' for output, which return a constant value of 'WAII'")])
        self.waii_client.semantic_context.modify_semantic_context(request)

    def create_semantic_rules_for_user1(self):
        with self.waii_client.impersonate_user("multi-tenant-semantic-context-test-user1"):
            request = ModifySemanticContextRequest(updated=[SemanticStatement(
                id="user1-semantic-statement-1",
                statement="All the returned query must project column 'FROM_USER' for output, which return a constant value of 'USER1'",
                user_id="multi-tenant-semantic-context-test-user1",
                tenant_id="test-tenant",
                org_id="multi-tenant-semantic-context-test")
            ])
            self.waii_client.semantic_context.modify_semantic_context(request)

    def create_semantic_rules_for_user2(self):
        with self.waii_client.impersonate_user("multi-tenant-semantic-context-test-user2"):
            request = ModifySemanticContextRequest(updated=[SemanticStatement(
                id="user2-semantic-statement",
                statement="All the returned query must project column 'FROM_USER' for output, which return a constant value of 'USER2'",
                user_id="multi-tenant-semantic-context-test-user2",
                tenant_id="test-tenant",
                org_id="multi-tenant-semantic-context-test")
            ])
            self.waii_client.semantic_context.modify_semantic_context(request)

    @staticmethod
    def _semantic_id_in_list(semantic_id, semantic_list):
        for semantic in semantic_list:
            if semantic.id == semantic_id:
                return True
        return False

    def test_list_semantic_statements(self):
        response = self.waii_client.semantic_context.get_semantic_context().semantic_context
        assert self._semantic_id_in_list("global-semantic-statement-1", response)

        with self.waii_client.impersonate_user("multi-tenant-semantic-context-test-user1"):
            response = self.waii_client.semantic_context.get_semantic_context().semantic_context
            assert self._semantic_id_in_list("user1-semantic-statement-1", response) and not self._semantic_id_in_list(
                "user2-semantic-statement", response)

        with self.waii_client.impersonate_user("multi-tenant-semantic-context-test-user2"):
            response = self.waii_client.semantic_context.get_semantic_context().semantic_context
            assert self._semantic_id_in_list("user2-semantic-statement", response) and not self._semantic_id_in_list(
                "user1-semantic-statement-1", response)

    def test_gen_query_uses_semantic_context(self):
        # generate query as main user
        response = self.waii_client.query.generate(QueryGenerationRequest(
            ask="how many movies are there"
        )).run()
        assert response.rows[0]['WAII_TEST'] == 'WAII'

        # generate query as user1
        with self.waii_client.impersonate_user("multi-tenant-semantic-context-test-user1"):
            response = self.waii_client.query.generate(QueryGenerationRequest(
                ask="how many movies are there"
            )).run()
            assert response.rows[0]['FROM_USER'] == 'USER1'
            assert response.rows[0]['WAII_TEST'] == 'WAII'

        # generate query as user2
        with self.waii_client.impersonate_user("multi-tenant-semantic-context-test-user2"):
            response = self.waii_client.query.generate(QueryGenerationRequest(
                ask="how many movies are there"
            )).run()
            assert response.rows[0]['FROM_USER'] == 'USER2'
            assert response.rows[0]['WAII_TEST'] == 'WAII'



import unittest

from tests.common_test_utils import org_exists, tenant_exists, user_exists
from waii_sdk_py.user import User as UserModel, BaseModel, DeleteTenantRequest, DeleteOrganizationRequest, \
    CreateOrganizationRequest, Organization, ListOrganizationsRequest, UpdateOrganizationRequest, CreateTenantRequest, \
    Tenant, ListTenantsRequest, UpdateTenantRequest
from waii_sdk_py import WAII
from waii_sdk_py.user import CreateAccessKeyRequest, DelAccessKeyRequest, DelAccessKeyResponse, GetAccessKeyRequest, \
    GetUserInfoRequest, GetUserInfoResponse, UpdateConfigRequest, CreateUserRequest, User, DeleteUserRequest, \
    CommonResponse, ListUsersRequest, UpdateUserRequest
from waii_sdk_py.waii_sdk_py import Waii


class TestUser(unittest.TestCase):
    def setUp(self):
        WAII.initialize(url="http://localhost:9859/api/")

    def test_create_access_key(self):
        params = DelAccessKeyRequest(names=["test1"])
        resp = WAII.User.delete_access_key(params)
        params = CreateAccessKeyRequest(name="test1")
        resp = WAII.User.create_access_key(params)
        test1_key = [key for key in resp.access_keys if key.name == "test1"]
        assert len(test1_key) > 0

    def test_delete_access_key(self):
        params = DelAccessKeyRequest(names=["test2"])
        resp = WAII.User.delete_access_key(params)
        params = CreateAccessKeyRequest(name="test2")
        resp = WAII.User.create_access_key(params)
        test2_key = [key for key in resp.access_keys if key.name == "test2"]
        assert len(test2_key) > 0

        params = GetAccessKeyRequest()
        resp = WAII.User.list_access_keys(params)
        test2_key = [key for key in resp.access_keys if key.name == "test2"]
        assert len(test2_key) > 0

        params = DelAccessKeyRequest(names=["test2"])
        resp = WAII.User.delete_access_key(params)
        assert isinstance(resp, DelAccessKeyResponse)

        params = GetAccessKeyRequest()
        resp = WAII.User.list_access_keys(params)
        test2_key = [key for key in resp.access_keys if key.name == "test2"]
        assert len(test2_key) == 0

    def test_list_access_keys(self):
        params = DelAccessKeyRequest(names=["test2"])
        resp = WAII.User.delete_access_key(params)

        params = GetAccessKeyRequest()
        resp = WAII.User.list_access_keys(params)
        test2_key = [key for key in resp.access_keys if key.name == "test2"]
        assert len(test2_key) == 0

        params = CreateAccessKeyRequest(name="test2")
        resp = WAII.User.create_access_key(params)
        test2_key = [key for key in resp.access_keys if key.name == "test2"]
        assert len(test2_key) > 0

        params = GetAccessKeyRequest()
        resp = WAII.User.list_access_keys(params)
        test2_key = [key for key in resp.access_keys if key.name == "test2"]
        assert len(test2_key) > 0

        params = DelAccessKeyRequest(names=["test2"])
        resp = WAII.User.delete_access_key(params)

    def test_get_user_info(self):
        params = GetUserInfoRequest()
        resp = WAII.User.get_user_info(params)
        assert isinstance(resp, GetUserInfoResponse)

    def test_update_config(self):
        params = UpdateConfigRequest(updated={"key1": "value1", "key2": "value2"})
        resp = WAII.User.update_config(params)
        assert resp.configs['key1'] == 'value1'
        assert resp.configs['key2'] == 'value2'
        params = UpdateConfigRequest(deleted=["key1"], updated={"key3": "value3"})
        resp = WAII.User.update_config(params)
        assert 'key1' not in resp.configs
        assert resp.configs['key3'] == 'value3'
        params = UpdateConfigRequest()
        resp = WAII.User.update_config(params)
        assert resp.configs['key2'] == 'value2'
        assert resp.configs['key3'] == 'value3'

    def test_manage_user(self):
        params = DeleteUserRequest(id="user1")
        try:
            resp = WAII.User.delete_user(params)
        except:
            pass
        params = CreateUserRequest(
            user=UserModel(id="user1", name="Wangda Tan", tenant_id="my_tenant_id", org_id="my_org_id"))
        resp = WAII.User.create_user(params)
        assert isinstance(resp, CommonResponse)

        params = ListUsersRequest(lookup_org_id="my_org_id")
        resp = WAII.User.list_users(params)
        user1 = [user for user in resp.users if user.id == "user1"]
        assert len(user1) > 0

        params = UpdateUserRequest(
            user=UserModel(id="user1", name="Pravin", tenant_id="my_tenant_id", org_id="my_org_id"))
        resp = WAII.User.update_user(params)
        assert isinstance(resp, CommonResponse)

        params = ListUsersRequest(lookup_org_id="my_org_id")
        resp = WAII.User.list_users(params)
        user1 = [user for user in resp.users if user.id == "user1"]
        assert user1[0].name == "Pravin"

        params = DeleteUserRequest(id="user1")
        resp = WAII.User.delete_user(params)
        assert isinstance(resp, CommonResponse)

        params = ListUsersRequest(lookup_org_id="my_org_id")
        resp = WAII.User.list_users(params)
        user1 = [user for user in resp.users if user.id == "user1"]
        assert len(user1) == 0

    def test_manage_org(self):
        params = DeleteOrganizationRequest(id="o1")
        try:
            resp = WAII.User.delete_org(params)
        except:
            pass
        params = CreateOrganizationRequest(organization=Organization(id="o1", name="My Org"))
        resp = WAII.User.create_org(params)
        assert isinstance(resp, CommonResponse)
        params = ListOrganizationsRequest()
        resp = WAII.User.list_orgs(params)
        org1 = [org for org in resp.organizations if org.id == "o1"]
        assert len(org1) > 0
        params = UpdateOrganizationRequest(organization=Organization(id="o1", name="My Org2"))
        resp = WAII.User.update_org(params)
        assert isinstance(resp, CommonResponse)
        params = ListOrganizationsRequest()
        resp = WAII.User.list_orgs(params)
        org1 = [org for org in resp.organizations if org.id == "o1"][0]
        assert org1.name == "My Org2"
        params = DeleteOrganizationRequest(id="o1")
        resp = WAII.User.delete_org(params)
        assert isinstance(resp, CommonResponse)
        params = ListOrganizationsRequest()
        resp = WAII.User.list_orgs(params)
        org1 = [org for org in resp.organizations if org.id == "o1"]
        assert len(org1) == 0

    def test_manage_tenant(self):
        params = CreateOrganizationRequest(organization=Organization(id="o1", name="My Org"))
        del_tenant = DeleteTenantRequest(id="tenant1")
        try:
            resp = WAII.User.create_org(params)
        except:
            pass
        try:
            resp = WAII.User.delete_tenant(del_tenant)
        except:
            pass
        params = CreateTenantRequest(tenant=Tenant(id="tenant1", name="Test Tenant", org_id="o1"))
        resp = WAII.User.create_tenant(params)
        assert isinstance(resp, CommonResponse)
        params = ListTenantsRequest(lookup_org_id="o1")
        resp = WAII.User.list_tenants(params)
        tenant1 = [tenant for tenant in resp.tenants if tenant.id == "tenant1"]
        assert len(tenant1) > 0
        params = UpdateTenantRequest(tenant=Tenant(id="tenant1", name="Test Tenant2", org_id="o1"))
        resp = WAII.User.update_tenant(params)
        assert isinstance(resp, CommonResponse)
        params = ListTenantsRequest(lookup_org_id="o1")
        resp = WAII.User.list_tenants(params)
        tenant1 = [tenant for tenant in resp.tenants if tenant.id == "tenant1"][0]
        assert tenant1.name == "Test Tenant2"
        params = DeleteTenantRequest(id="tenant1")
        resp = WAII.User.delete_tenant(params)
        assert isinstance(resp, CommonResponse)
        params = ListTenantsRequest()
        resp = WAII.User.list_tenants(params)
        tenant1 = [tenant for tenant in resp.tenants if tenant.id == "tenant1"]
        assert len(tenant1) == 0

    def test_impersonation(self):
        client = Waii()
        client.initialize(url="http://localhost:9859/api/")

        # create an hierarchy of organizations, tenants and users
        if not org_exists(client, "org1"):
            org = Organization(id="org1", name="My Org")
            WAII.User.create_org(CreateOrganizationRequest(organization=org))

        if not tenant_exists(client, "org1", "tenant1"):
            tenant = Tenant(id="tenant1", name="Test Tenant", org_id="org1")
            WAII.User.create_tenant(CreateTenantRequest(tenant=tenant))

        if not user_exists(client, "org1", "user1"):
            user = User(id="user1", name="Wangda Tan", tenant_id="tenant1", org_id="org1")
            WAII.User.create_user(CreateUserRequest(user=user))

        if not user_exists(client, "org1", "user2"):
            user = User(id="user2", name="Pravin", tenant_id="tenant1", org_id="org1")
            WAII.User.create_user(CreateUserRequest(user=user))

        # impersonate the user
        client.set_impersonate_user("user1")

        # get the user info
        resp = client.user.get_user_info(GetUserInfoRequest())
        assert isinstance(resp, GetUserInfoResponse)
        assert resp.id == "user1"
        assert resp.name == "Wangda Tan"

        # test impersonation, with block
        with client.impersonate_user("user1"):
            resp = client.user.get_user_info(GetUserInfoRequest())
            assert isinstance(resp, GetUserInfoResponse)
            assert resp.id == "user1"
            assert resp.name == "Wangda Tan"

        resp = client.user.get_user_info(GetUserInfoRequest())
        assert isinstance(resp, GetUserInfoResponse)
        assert resp.id == "my_org_id"
        assert resp.name == "J.A.R.V.I.S."

        with client.impersonate_user("user2"):
            resp = client.user.get_user_info(GetUserInfoRequest())
            assert isinstance(resp, GetUserInfoResponse)
            assert resp.id == "user2"
            assert resp.name == "Pravin"


if __name__ == '__main__':
    unittest.main()



from typing import Optional, List

from ..my_pydantic import BaseModel


class CommonRequest(BaseModel):
    tags: Optional[List[str]]


class LLMBasedRequest(CommonRequest):
    model: Optional[str]
    # should we use cache?
    use_cache: Optional[bool] = True


class CommonResponse(BaseModel):
    pass



import unittest

from tests.common_test_utils import connect_db, load_db_conn1, check_table_existence
from waii_sdk_py import WAII
from waii_sdk_py.database import (
    ModifyDBConnectionRequest,
    DBConnection,
    TableDefinition,
    TableName,
    ColumnDefinition,
    TableReference,
    UpdateTableDefinitionRequest, )
from waii_sdk_py.query import RunQueryRequest


class TestDatabase(unittest.TestCase):
    def _load_push_based_conn(self):
        self.push_db_conn = DBConnection(
            key="",  # will be generated by the system
            db_type="snowflake",
            database="push",
            host="push_sdk_test",
            push=True,
        )

    def setUp(self):
        WAII.initialize(url="http://localhost:9859/api/")

        self.db_conn = load_db_conn1()
        self._load_push_based_conn()

        connect_db(self.db_conn)

    def test_modify_connections(self):
        # first try to delete te connection if it exists
        result = WAII.Database.get_connections().connectors
        for conn in result:
            if (
                    conn.db_type == self.db_conn.db_type
                    and conn.account_name == self.db_conn.account_name
                    and self.db_conn.database == conn.database
                    and self.db_conn.username == conn.username
            ):
                result = WAII.Database.modify_connections(
                    ModifyDBConnectionRequest(removed=[conn.key])
                ).connectors
                break

        # then add the new connection
        new_result = WAII.Database.modify_connections(
            ModifyDBConnectionRequest(updated=[self.db_conn])
        ).connectors

        assert len(result) == len(new_result) - 1

        # then activate the new connection
        for conn in new_result:
            if (
                    conn.db_type == self.db_conn.db_type
                    and conn.account_name == self.db_conn.account_name
                    and self.db_conn.database == conn.database
                    and self.db_conn.username == conn.username
            ):
                WAII.Database.activate_connection(conn.key)
                break
        else:
            raise Exception("Cannot find the new connection")

        # list databases
        result = WAII.Database.get_catalogs()
        assert len(result.catalogs) > 0
        for catalog in result.catalogs:
            for schema in catalog.schemas:
                for table in schema.tables:
                    if len(table.refs) > 0:
                        assert type(table.refs[0]) == TableReference

    def test_modify_push_connection(self):
        result = WAII.Database.get_connections().connectors
        for conn in result:
            if (
                    conn.db_type == self.push_db_conn.db_type
                    and conn.host == self.push_db_conn.host
                    and conn.push
            ):
                result = WAII.Database.modify_connections(
                    ModifyDBConnectionRequest(removed=[conn.key])
                ).connectors
                break
        new_result = WAII.Database.modify_connections(
            ModifyDBConnectionRequest(updated=[self.push_db_conn])
        ).connectors

        assert len(result) == len(new_result) - 1

    def test_get_connections(self):
        # Call the function
        result = WAII.Database.get_connections()
        # Check the result
        # Note: The specifics of this assertion would depend on what the function should return
        self.assertEqual(len(result.connectors), len(result.connectors))

        # Define test parameters
        params = ModifyDBConnectionRequest(updated=[], removed=[])
        # Call the function
        result = WAII.Database.modify_connections(params)
        # Check the result
        # Note: The specifics of this assertion would depend on what the function should return
        self.assertEqual(len(result.connectors), len(result.connectors))

    def test_activate_connection(self):
        # Define test parameters
        # Call the function
        WAII.Database.activate_connection(self.db_conn.key)

    def test_get_catalogs(self):
        # Call the function
        result = WAII.Database.get_catalogs()
        assert len(result.catalogs) > 0

    def test_get_refs_from_api(self):
        result = WAII.Database.get_catalogs()
        catalog_def = result.catalogs[0]
        schemas = catalog_def.schemas
        refs = []
        for schema in schemas:
            if schema.name.schema_name == "INFORMATION_SCHEMA":
                continue
            for table in schema.tables:
                refs.extend(table.refs)

    def test_get_refs(self):
        table_definition = TableDefinition(
            name=TableName(
                database_name="db1", schema_name="schema1", table_name="table1"
            ),
            columns=[
                ColumnDefinition(name="col1", type="int"),
                ColumnDefinition(name="col2", type="int"),
                ColumnDefinition(name="col3", type="int"),
            ],
            refs=[
                TableReference(
                    src_table=TableName(
                        database_name="db1", schema_name="schema1", table_name="table1"
                    ),
                    src_cols=["col3"],
                    ref_table=TableName(
                        database_name="db1", schema_name="schema1", table_name="table1"
                    ),
                    ref_cols=["col3"],
                ),
            ],
        )
        assert len(table_definition.refs) > 0
        assert table_definition.refs[0] == TableReference(
            src_table=TableName(
                database_name="db1", schema_name="schema1", table_name="table1"
            ),
            src_cols=["col3"],
            ref_table=TableName(
                database_name="db1", schema_name="schema1", table_name="table1"
            ),
            ref_cols=["col3"],
        )

    def test_initial_connect(self):
        # because now we select first connection by default
        result = WAII.Database.get_catalogs()
        assert len(result.catalogs) > 0

        assert len(WAII.Database.get_activated_connection()) > 0

    def test_call_without_activated_connection(self):
        WAII.Database.activate_connection("")
        with self.assertRaises(Exception):
            result = WAII.Database.get_catalogs()

    def test_tabl_ref_deserialize(self):
        table_dict = {
            "name": TableName(
                table_name="table1", schema_name="schema1", database_name="db1"
            ),
            "columns": [
                ColumnDefinition(
                    name="col1",
                    type="int",
                    comment=None,
                    description=None,
                    sample_values=None,
                ),
                ColumnDefinition(
                    name="col2",
                    type="int",
                    comment=None,
                    description=None,
                    sample_values=None,
                ),
                ColumnDefinition(
                    name="col3",
                    type="int",
                    comment=None,
                    description=None,
                    sample_values=None,
                ),
            ],
            "comment": None,
            "last_altered_time": None,
            "constraints": None,
            "inferred_refs": None,
            "inferred_constraints": None,
            "description": None,
            "refs": [
                {
                    "ref_cols": ["col3"],
                    "ref_table": {
                        "database_name": "db1",
                        "schema_name": "schema1",
                        "table_name": "table1",
                    },
                    "source": None,
                    "src_cols": ["col3"],
                    "src_table": {
                        "database_name": "db1",
                        "schema_name": "schema1",
                        "table_name": "table1",
                    },
                }
            ],
        }
        ref_dict = {
            "src_table": {
                "table_name": "MATCHES",
                "schema_name": "WTA_1",
                "database_name": "SPIDER_DEV",
            },
            "src_cols": ["WINNER_ID"],
            "ref_table": {
                "table_name": "PLAYERS",
                "schema_name": "WTA_1",
                "database_name": "SPIDER_DEV",
            },
            "ref_cols": ["PLAYER_ID"],
            "source": "database",
            "score": None,
        }
        table_def = TableDefinition(**table_dict)
        assert type(table_def.refs[0]) == TableReference
        assert table_def.refs[0] == TableReference(
            src_table=TableName(
                database_name="db1", schema_name="schema1", table_name="table1"
            ),
            src_cols=["col3"],
            ref_table=TableName(
                database_name="db1", schema_name="schema1", table_name="table1"
            ),
            ref_cols=["col3"],
        )

    def test_update_table_definitions(self):

        table_definition = TableDefinition(
            name=TableName(
                database_name="db1", schema_name="schema2", table_name="table2"
            ),
            columns=[
                ColumnDefinition(name="col1", type="int"),
                ColumnDefinition(name="col2", type="int"),
                ColumnDefinition(name="col3", type="int"),
            ],
        )
        update_table_req = UpdateTableDefinitionRequest(
            updated_tables=[table_definition]
        )
        result = WAII.Database.update_table_definition(update_table_req)

    def test_remove_tables(self):
        table_to_be_removed = TableName(
            database_name="db1", schema_name="schema2", table_name="table2"
        )

        update_table_req = UpdateTableDefinitionRequest(
            removed_tables=[table_to_be_removed]
        )
        result = WAII.Database.update_table_definition(update_table_req)

    def test_refresh_db_connection(self):
        WAII.Database.activate_connection(self.db_conn.key)

        # get catalog and see if test_refresh_db_connection table exists
        result = WAII.Database.get_catalogs()
        table_exists = False
        for catalog in result.catalogs:
            for schema in catalog.schemas:
                for table in schema.tables:
                    if table.name.table_name.lower() == "test_refresh_db_connection":
                        table_exists = True
                        break

        # if table exists, run DROP TABLE
        if table_exists:
            try:
                WAII.Query.run(RunQueryRequest(query="DROP TABLE test_refresh_db_connection_schema.test_refresh_db_connection"))
            except Exception as e:
                print(f"Error dropping table: {e}")

            table_removed = check_table_existence(
                WAII, "test_refresh_db_connection", False
            )

            assert table_removed

        # then create a new table
        WAII.Query.run(
            RunQueryRequest(
                query="CREATE SCHEMA IF NOT EXISTS test_refresh_db_connection_schema"
            )
        )

        WAII.Query.run(
            RunQueryRequest(
                query="CREATE TABLE test_refresh_db_connection_schema.test_refresh_db_connection (id INT)"
            )
        )

        table_created = check_table_existence(
            WAII, "test_refresh_db_connection", True
        )

        assert table_created


# NEED TO ADD FOR UPDATE TABLE AND UPDATE SCHEMA
if __name__ == "__main__":
    unittest.main()



try:
    from pydantic.v1 import (
        BaseModel,
        ValidationError,
        PrivateAttr,
        Field,
        # Add other necessary imports here
    )
except ImportError:
    try:
        from pydantic import (
            BaseModel,
            ValidationError,
            PrivateAttr,
            Field,
            # Add other necessary imports here
        )
    except ImportError:
        raise ImportError("Cannot find pydantic module. Please install pydantic. You can use >= 1.10.x or >= 2.7.x")

__all__ = [
    "BaseModel",
    "ValidationError",
    "PrivateAttr",
    "Field"
]